#!/bin/env python
# -*- coding: utf-8 -*-

"""
Interaktiv visualisering av specialiseringar på civilingenjörsprogram
inom Lunds universitet. Avsikten är att tydliggöra vilka institutioner
som ger vilka kurser inom vilka specialiseringar.

Data hämtas från Läro- och timwebben, http://www.ceq.lth.se/LoTwebb,
och lagras i separata textfiler för varje program. I en framtida
version kan detta komma att ersättas av data hämtade via webben.

Daniel Sjoberg, 2011-05-28
"""

import wx

# Keep the same RGB colour on different platforms
#AttentionColour = wx.Colour(200, 200, 200)
#AttentionColour2 = wx.Colour(255, 255, 255)
#AttentionColour = wx.Colour(255, 165, 0)
#AttentionColour2 = wx.Colour(255, 200, 0)
AttentionColour = wx.Colour(127, 255, 0)
AttentionColour2 = wx.Colour(180, 255, 120)

ProgramList = [
               u'Bioteknik',
               u'Datateknik',
               u'Ekosystemteknik',
               u'Elektroteknik',
               u'Industriell ekonomi',
               u'Informations- och kommunikationsteknik',
               u'Kemiteknik',
               u'Lantmäteri',
               u'Maskinteknik',
               u'Teknisk fysik', 
               u'Teknisk matematik',
               u'Teknisk nanovetenskap',
               u'Väg- och vattenbyggnad',
               ]

class KAData():
    """
    This class contains the data for the courses, as found in the KA
    system.
    """
    def __init__(self, Program):
        """
        Initialize the KA data, which is read from a text file dumped
        from the LoT web page.
        """
        self.Specialiseringar, self.Kurser = self.ReadDataBaseFromLoT(Program)
        self.Institutioner = {}
        self.Institutioner['fys'] = InstData(u'Fysiska institutionen',
                        ['FKF', 'FAF', 'FFF', 'FBR', 'FMF', 'TEK177', 'EXTF05',
                         'TEK265', 'TEK267'])
        self.Institutioner['bygg'] = InstData(u'Byggvetenskaper',
                                     ['VBE', 'ADP', 'FHL', 'VTA', 'VSM', 'VGT',
                                      'AFO280', 'GEMA40', 'FME602', 'MAMF25', 
                                      'EDA602', 'EDAA15'])
        self.Institutioner['cs'] = InstData(u'Datavetenskap',
                                          ['EDA', 'ETS032', 'ETS170', 'ETS200',
                                           'ETSN05'])
        self.Institutioner['des'] = InstData(u'Designvetenskaper',
                                             ['MAM', 'MMK', 'TNX097', 'TNX153'])
        self.Institutioner['elmat'] = InstData(u'Elektrisk mätteknik och IEA',
                                         ['EEM', 'EIE', 'MIE', 'VTG', 'GEMA55',
                                          'ESSF15', 'ESSF10'])
        self.Institutioner['eit'] = InstData(u'Elektro- och informationsteknik',
                                     ['EIT', 'ETE', 'ETI', 'ETT', 'EDI', 'ESS',
                                      'ETS061', 'ETS075', 'ETS110', 'ETS190',
                                      'ETSN01', 'ETSF10'])
        self.Institutioner['energy'] = InstData(u'Energivetenskaper',
                                                ['MVK', 'MMV'])
        self.Institutioner['mtek'] = InstData(u'Maskinteknologi',
                                              ['FME', 'FKM', 'MMT', 'MME'])
        self.Institutioner['mc'] = InstData(u'Matematikcentrum',
                                            ['FMA', 'FMS', 'FMN'])
        self.Institutioner['regler'] = InstData(u'Reglerteknik',
                                                ['FRT'])
        self.Institutioner['tos'] = InstData(u'Teknik och samhälle',
                    ['VVB', 'VFR', 'FMI', 'VFT', 'VGM', 'VTT', 'VTV', 'GEMF01'])
        self.Institutioner['abm'] = InstData(u'Arkitektur och byggd miljö',
                                      ['ABV', 'ABA', 'AEB', 'AAH', 'AHI', 'ATE',
                                       'ASB', 'AFO', 'AAM', 'AAK', 'ABF'])
        self.Institutioner['bmt'] = InstData(u'Bygg- och miljöteknologi',
                                     ['VBR', 'VBF', 'VBM', 'ABK', 'VBK', 'VVR'])
        self.Institutioner['imt'] = InstData(u'Immunteknologi', ['KIM'])
        self.Institutioner['kt'] = InstData(u'Kemiteknik', 
                                         ['KTE', 'VVA', 'KET', 'KAT', 'GEMA45'])
        self.Institutioner['lmt'] = InstData(u'Livsmedelsteknik',
                     ['KNL', 'KLG', 'BLT', 'KLT', 'KFK025', 'KAT080', 'FMS210'])
        self.Institutioner['tel'] = InstData(u'Teknisk ekonomi och logistik',
                                             ['MIO', 'MTT'])
        self.Institutioner['kem'] = InstData(u'Kemiska institutionen',
                                     ['KFK', 'KOK', 'KBT', 'KKK', 'KOO', 'KPO', 
                                      'KTE080', 'KAK', 'KMB', 'KBK'])
        self.Institutioner['zzz'] = InstData(u'Annan fakultet',
                                             ['TEK', 'INN', 'EXT', 'KII010'])


    def ReadDataBaseFromLoT(self, Program):
        """Read the course data base from a text dump of the LoT web."""
        Specialiseringar = {}
        Kurser = {}
        if ProgramList[Program] == u'Teknisk fysik':
            import lotdata1112_f
            lines = lotdata1112_f.data.splitlines()
        elif ProgramList[Program] == u'Elektroteknik':
            import lotdata1112_e
            lines = lotdata1112_e.data.splitlines()
        elif ProgramList[Program] == u'Datateknik':
            import lotdata1112_d
            lines = lotdata1112_d.data.splitlines()
        elif ProgramList[Program] == u'Informations- och kommunikationsteknik':
            import lotdata1112_c
            lines = lotdata1112_c.data.splitlines()
        elif ProgramList[Program] == u'Bioteknik':
            import lotdata1112_b
            lines = lotdata1112_b.data.splitlines()
        elif ProgramList[Program] == u'Industriell ekonomi':
            import lotdata1112_i
            lines = lotdata1112_i.data.splitlines()
        elif ProgramList[Program] == u'Kemiteknik':
            import lotdata1112_k
            lines = lotdata1112_k.data.splitlines()
        elif ProgramList[Program] == u'Lantmäteri':
            import lotdata1112_l
            lines = lotdata1112_l.data.splitlines()
        elif ProgramList[Program] == u'Maskinteknik':
            import lotdata1112_m
            lines = lotdata1112_m.data.splitlines()
        elif ProgramList[Program] == u'Teknisk nanovetenskap':
            import lotdata1112_n
            lines = lotdata1112_n.data.splitlines()
        elif ProgramList[Program] == u'Teknisk matematik':
            import lotdata1112_pi
            lines = lotdata1112_pi.data.splitlines()
        elif ProgramList[Program] == u'Väg- och vattenbyggnad':
            import lotdata1112_v
            lines = lotdata1112_v.data.splitlines()
        elif ProgramList[Program] == u'Ekosystemteknik':
            import lotdata1112_w
            lines = lotdata1112_w.data.splitlines()
        else:
            import lotdata1112_f
            lines = lotdata1112_f.data.splitlines()
            

        N = len(lines)
        k = 0
        while k < N:
            # Find specialization
            while k < N:
                line = lines[k]
                k = k + 1
                test = line.split()
                try: 
                    if test[0] == 'Specialisering':
                        SpecKey = test[1]
                        SpecName = line.split('- ', 1)[1][0:-1]
                        Specialiseringar[SpecKey] = SpecName
                        break
                    else:
                        continue
                except:
                    continue
        # Read courses in specialization
            while k < N:
                line = lines[k]
                k = k + 1
                test = line.split('\t')
                try:
                    if test[2] in ('G1', 'G2', 'A'):
                        KursKod = test[0]
                        KursNamn = test[8]
                    # Check if course is already stored, if so, only add spec
                        if not KursKod in Kurser:
                            Kurser[KursKod] = KursData(KursKod, KursNamn, [SpecKey])
                        else:
                            Kurser[KursKod].ProgramKoppling.append(SpecKey)
                except:
                    test = line.split()
                    try:
                        if test[0] == 'Specialisering':
                            k = k - 1   # Back one line
                            break
                        elif test[0] == 'Valfria':
                            break
                    except:
                        continue
                    continue
        return(Specialiseringar, Kurser)


    def GetSpecKurs(self, Kurs):
        """Return a list of specializations where the course appears."""
        return(self.Kurser[Kurs].ProgramKoppling)

    def GetInstKurs(self, Kurs):
        """Return the department where the course is given."""
        ReturnInst = None
        # First check for complete course code
        for Inst in self.Institutioner:
            if Kurs in self.Institutioner[Inst].KursKoder:
                ReturnInst = Inst
        if not ReturnInst:
            for Inst in self.Institutioner:
                if Kurs[0:3] in self.Institutioner[Inst].KursKoder:
                    ReturnInst = Inst
        if not ReturnInst:
            ReturnInst = 'zzz'
        return(ReturnInst)

    def GetKurserInst(self, Inst):
        """Return a list of courses given at a department."""
        KursLista = {}
        for Kurs in self.Kurser:
            KursKod = self.Kurser[Kurs].KursKod
            # Check if full kurskod is in another department's list
            UniqueInst = True
            for InstKey in self.Institutioner:
                if KursKod in self.Institutioner[InstKey].KursKoder:
                    if InstKey != Inst:
                        UniqueInst = False
                    else:
                        KursLista[KursKod] = self.Kurser[Kurs].KursNamn
            # Then check generic codes
            if UniqueInst:
                if KursKod[0:3] in self.Institutioner[Inst].KursKoder:
                    KursLista[KursKod] = self.Kurser[Kurs].KursNamn
        return(KursLista)

    def GetKurserSpec(self, Spec):
        """Return a list of courses in a specialization."""
        KursLista = {}
        for Kurs in self.Kurser:
            if Spec in self.Kurser[Kurs].ProgramKoppling:
                KursLista[Kurs] = self.Kurser[Kurs].KursNamn
        return(KursLista)

    def GetInst(self):
        """Return a list of departments."""
        InstLista = {}
        for Inst in self.Institutioner:
            InstLista[Inst] = self.Institutioner[Inst].InstNamn
        return(InstLista)

    def GetSpec(self):
        """Return a list of specializations."""
        SpecLista = {}
        for Spec in self.Specialiseringar:
            SpecLista[Spec] = self.Specialiseringar[Spec]
        return(SpecLista)

class KursData():
    """This class defines the data structure for each individual course."""
    def __init__(self, KursKod, KursNamn, ProgramKoppling):
        """Initialize the course data."""
        self.KursKod = KursKod
        self.KursNamn = KursNamn
        self.ProgramKoppling = ProgramKoppling

class InstData():
    """This class defines the data structure for each individual department."""
    def __init__(self, InstNamn, KursKoder):
        self.InstNamn = InstNamn
        self.KursKoder = KursKoder
                                                      

class MyFrame(wx.Frame):
    """This class holds the main GUI window."""
    def __init__(self, parent, title):
        """Build the GUI."""
        wx.Frame.__init__(self, parent, -1, title)
        self.SetBackgroundColour('Normal')

        MenuBar = wx.MenuBar()
        FileMenu = wx.Menu()
        item = FileMenu.Append(wx.ID_EXIT, text="&Quit")
        self.Bind(wx.EVT_MENU, self.OnQuit, item)
        HelpMenu = wx.Menu()
        item = HelpMenu.Append(wx.ID_HELP, text="&About")
        self.Bind(wx.EVT_MENU, self.OnAbout, item)
        MenuBar.Append(FileMenu, "&File")
        MenuBar.Append(HelpMenu, "&Help")
        self.SetMenuBar(MenuBar)

        self.ScrollWindow = wx.ScrolledWindow(self)
        tp = TopPanel(self.ScrollWindow)
        tp.QuitPanel.Bind(wx.EVT_BUTTON, self.OnQuit)
        width, height = tp.GetSize()
        dpixel = 20
        self.ScrollWindow.SetScrollbars(dpixel, dpixel, 
                                        round(width/dpixel), 
                                        round(height/dpixel))

    def OnQuit(self, event=None):
        """End the GUI."""
        ret  = wx.MessageBox(u'Är du säker att du vill avsluta?', 'Question', 
                             wx.YES_NO | wx.NO_DEFAULT, self)
        if ret == wx.YES:
            self.Close()

    def OnAbout(self, event=None):
        """Display some background about the program."""
        ret  = wx.MessageBox(u"""
Detta program illustrerar kopplingar mellan specialiseringar, kurser och institutioner. 

Klicka på specialisering för att se vilka kurser som ingår och vilka institutioner som ger dem.

Klicka på kurs för att se vilken institution som ger den och i vilka specialiseringar den ingår.

Klicka på institution för att se vilka kurser den ger och i vilka specialiseringar de ingår.

Data är tagna 2011-05-28 från Läro- och timplanwebben för läsår 11/12, mindre förändringar kan ha skett. Se http://www.ceq.lth.se/LoTwebb för slutliga kurslistor, detta program är endast ett komplement.

Programmet är experimentellt och inte helt avlusat. Exempelvis kan rubriker hamna fel om man valt ett program och sedan väljer samma program igen, vilket ordnar sig om man väljer ett nytt program. Om det beter sig underligt på andra sätt, testa att starta om. Om du vill bidra med åsikter och lösningar, hör gärna av dig enligt nedan.

Daniel Sjöberg, daniel.sjoberg@eit.lth.se, 2011-05-28.""", 
                             'Info', wx.OK, self)

class TopPanel(wx.Panel):
    """The top panel of the GUI."""
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        self.parent = parent

        self.ProgChoice = wx.Choice(self, -1, choices = ProgramList)
        self.ProgChoice.Bind(wx.EVT_CHOICE, self.OnProgChoice)
        self.ProgChoice.SetSelection(ProgramList.index(u'Teknisk fysik'))
        self.Program = self.ProgChoice.GetSelection()

        self.QuitPanel = QuitPanel(self)
        self.InfoPanel = InfoPanel(self, self.Program)
#        self.ConnectionCB = ConnectionBox(self)

        self.vSizer = wx.BoxSizer(wx.VERTICAL)
        self.vSizer.Add(self.ProgChoice, 0, wx.ALIGN_LEFT | wx.ALL, 10)
        self.vSizer.Add(self.InfoPanel, 0, wx.ALIGN_CENTER | wx.ALL, 10)
#        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
#        self.hSizer.Add(self.ConnectionCB, 0, wx.ALIGN_CENTER | wx.ALL, 10)
#        self.hSizer.Add(self.QuitPanel, 0, wx.ALIGN_CENTER | wx.ALL, 10)
#        self.vSizer.Add(self.hSizer, 0, wx.ALIGN_CENTER | wx.ALL, 10)
        self.vSizer.Add(self.QuitPanel, 0, wx.ALIGN_CENTER | wx.ALL, 10)
        self.SetSizerAndFit(self.vSizer)

    def OnProgChoice(self, event):
        """Update the window for selected program."""
        self.InfoPanel.Destroy()
        self.Program = self.ProgChoice.GetSelection()
        self.InfoPanel = InfoPanel(self, self.Program)

        self.vSizer = wx.BoxSizer(wx.VERTICAL)
        self.vSizer.Add(self.ProgChoice, 0, wx.ALIGN_LEFT | wx.ALL, 10)
        self.vSizer.Add(self.InfoPanel, 0, wx.ALIGN_CENTER | wx.ALL, 10)
#        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
#        self.hSizer.Add(self.ConnectionCB, 0, wx.ALIGN_CENTER | wx.ALL, 10)
#        self.hSizer.Add(self.QuitPanel, 0, wx.ALIGN_CENTER | wx.ALL, 10)
#        self.vSizer.Add(self.hSizer, 0, wx.ALIGN_CENTER | wx.ALL, 10)
        self.vSizer.Add(self.QuitPanel, 0, wx.ALIGN_CENTER | wx.ALL, 10)
        self.SetSizerAndFit(self.vSizer)

        # Resize scrollbars in parent ScrollWindow
        width, height = self.GetSize()
        dpixel = 20
        self.parent.SetScrollbars(dpixel, dpixel, 
                                  round(width/dpixel), 
                                  round(height/dpixel))
        event.Skip()

class InfoPanel(wx.Panel):
    """Create the panel with course information."""
    def __init__(self, parent, Program):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.InfoPanel = self
        self.kursdatabas = KAData(Program)
        self.SortType = 'Spec'
        self.SortKey = sorted(self.kursdatabas.GetSpec().keys())[0]
        self.DrawConnections = True

        self.RowHeightSpec = self.GetCharHeight()*2 + 5
        self.RowHeightKurs = self.GetCharHeight() + 3
        self.RowHeightInst = self.GetCharHeight()*2 + 5
        self.RowWidthSpec = 250
        self.RowWidthKurs = 350
        self.RowWidthInst = 250


        # Calculate max number of courses
        MaxKurser = 0  
        for Inst in self.kursdatabas.GetInst():
            MaxKurser = max(MaxKurser,len(self.kursdatabas.GetKurserInst(Inst)))
        for Spec in self.kursdatabas.GetSpec():
            MaxKurser = max(MaxKurser,len(self.kursdatabas.GetKurserSpec(Spec)))
        self.MaxKurser = MaxKurser

        self.SpecList = ListPanel(self, 'Specialiseringar', 
                                  self.kursdatabas.GetSpec(), 'Spec')
        # Extract list of departments for this program
        InstList = {}  
        for Inst in self.kursdatabas.GetInst():
            for Spec in self.kursdatabas.GetSpec():
                for Kurs in self.kursdatabas.GetKurserSpec(Spec):
                    if Inst in self.kursdatabas.GetInstKurs(Kurs):
                        InstList[Inst] = self.kursdatabas.GetInst()[Inst]
        self.InstList = ListPanel(self, 'Institutioner', InstList, 'Inst')

        self.ConnectionPanel1 = ConnectionPanel(self, 'KursSpec')
        self.ConnectionPanel2 = ConnectionPanel(self, 'KursInst')

        self.CreateContent()

    def CreateContent(self):
        """
        Create the content. This consists in updating the course list,
        and setting the background colour of selected
        specializations/departments.
        """
        Key = self.SortKey
        self.ConnectionPanel1.Connections = []
        self.ConnectionPanel2.Connections = []
        if self.SortType == 'Inst':
            self.KursList = ListPanel(self, 'Kurser',
                                      self.kursdatabas.GetKurserInst(Key),
                                      'Kurs')
            self.InstList.Lista[Key].SetBackgroundColour(AttentionColour)
            for Spec in self.SpecList.Lista:
                for Kurs in self.KursList.Lista:
                    if Spec in self.kursdatabas.GetSpecKurs(Kurs):
                        self.SpecList.Lista[Spec].SetBackgroundColour(AttentionColour)
                        self.ConnectionPanel1.Connections.append([Kurs, Spec])
        elif self.SortType == 'Kurs':
            Kurs = Key
            Inst = self.kursdatabas.GetInstKurs(Kurs)
            KursNamn = self.kursdatabas.GetKurserInst(Inst)[Kurs]
            self.KursList = ListPanel(self, 'Kurser', 
                                      {Kurs: KursNamn},
                                      'Kurs')
            self.InstList.Lista[Inst].SetBackgroundColour(AttentionColour)
            for Spec in self.SpecList.Lista:
                if Spec in self.kursdatabas.GetSpecKurs(Kurs):
                    self.SpecList.Lista[Spec].SetBackgroundColour(AttentionColour)
                    self.ConnectionPanel1.Connections.append([Kurs, Spec])
        else:
            self.KursList = ListPanel(self, 'Kurser',
                                      self.kursdatabas.GetKurserSpec(Key),
                                      'Kurs')
            self.SpecList.Lista[Key].SetBackgroundColour(AttentionColour)
            for Inst in self.InstList.Lista:
                for Kurs in self.KursList.Lista:
                    InstLista = self.kursdatabas.GetInstKurs(Kurs)
                    if InstLista:  # Maybe the list of departments is empty
                        if Inst in InstLista:
                            self.InstList.Lista[Inst].SetBackgroundColour(AttentionColour)
                            self.ConnectionPanel2.Connections.append([Kurs, Inst])
        EvenOdd = 1
        for Kurs in sorted(self.KursList.Lista):
            if bool(EvenOdd%2):
	            self.KursList.Lista[Kurs].SetBackgroundColour(AttentionColour)
            else:
	            self.KursList.Lista[Kurs].SetBackgroundColour(AttentionColour2)
            EvenOdd = EvenOdd + 1
        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
        if self.DrawConnections:
            self.hSizer.Add(self.SpecList, 0, wx.ALIGN_TOP | wx.ALL, 0)
            self.hSizer.Add(self.ConnectionPanel1, 0, wx.ALIGN_TOP | wx.ALL, 0)
            self.hSizer.Add(self.KursList, 0, wx.ALIGN_TOP | wx.ALL, 0)
            self.hSizer.Add(self.ConnectionPanel2, 0, wx.ALIGN_TOP | wx.ALL, 0)
            self.hSizer.Add(self.InstList, 0, wx.ALIGN_TOP | wx.ALL, 0)
        else:
            self.hSizer.Add(self.SpecList, 0, wx.ALIGN_TOP | wx.ALL, 10)
            self.hSizer.Add(self.KursList, 0, wx.ALIGN_TOP | wx.ALL, 10)
            self.hSizer.Add(self.InstList, 0, wx.ALIGN_TOP | wx.ALL, 10)
        self.SetSizerAndFit(self.hSizer)

    def ClearContent(self):
        """
        Clear the content of the window. This consists in deleting the
        list of courses, and clearing the background colour on the
        list of specializations and departments.
        """
        if self.SortType != 'Kurs':
            self.KursList.Destroy()
        else:
            for Kurs in self.KursList.Lista:
                if Kurs != self.SortKey:
                    self.KursList.Lista[Kurs].Destroy()
            self.KursList.Fit()
        for Spec in self.SpecList.Lista:
            self.SpecList.Lista[Spec].SetBackgroundColour('NORMAL')
        for Inst in self.InstList.Lista:
            self.InstList.Lista[Inst].SetBackgroundColour('NORMAL')

    def MyUpdate(self):
        """Update the panel."""
        self.ClearContent()
        self.CreateContent()
        self.Refresh()  # Necessary for colours to be updated
#        self.parent.Fit()
        # The code below seems to be necessary for MSW to redraw
        event = wx.SizeEvent(self.GetSize())
        self.ProcessEvent(event)
        event.Skip()

class ConnectionPanel(wx.Panel):
    """
    Draw connections between courses and specializations and
    departments.
    """
    def __init__(self, parent, Direction):
        self.InfoPanel = parent.InfoPanel
        self.width = 50
        SpecHeight = self.InfoPanel.RowHeightSpec*(len(self.InfoPanel.SpecList.Lista)+1)
        KursHeight = self.InfoPanel.RowHeightKurs*(self.InfoPanel.MaxKurser+2)
        InstHeight = self.InfoPanel.RowHeightInst*(len(self.InfoPanel.InstList.Lista)+1)
        self.height = max(SpecHeight, KursHeight, InstHeight)
        wx.Panel.__init__(self, parent, size=(self.width, self.height))
        self.parent = parent
        self.Bind(wx.EVT_PAINT, self.OnPaint)

        self.Connections = []
        self.Direction = Direction
        Title = self.InfoPanel.SpecList.Title
        Height = Title.GetCharHeight()
        self.yOffset = Height

    def OnPaint(self, event):
        self.dc = wx.PaintDC(self)
        if self.Direction == 'KursInst':
            for Conn in self.Connections:
                KursKey = Conn[0]
                InstKey = Conn[1]
                y1idx = sorted(self.InfoPanel.KursList.Lista).index(KursKey)
                y2idx = sorted(self.InfoPanel.InstList.Lista).index(InstKey)
                x1 = 0
                y1 = (y1idx + 0.5)*self.InfoPanel.RowHeightKurs + self.yOffset
                x2 = self.width
                y2 = (y2idx + 0.5)*self.InfoPanel.RowHeightInst + self.yOffset
                self.dc.DrawLine(x1, y1, x2, y2)
        elif self.Direction == 'KursSpec':
            for Conn in self.Connections:
                KursKey = Conn[0]
                SpecKey = Conn[1]
                y1idx = sorted(self.InfoPanel.SpecList.Lista).index(SpecKey)
                y2idx = sorted(self.InfoPanel.KursList.Lista).index(KursKey)
                x1 = 0
                y1 = (y1idx + 0.5)*self.InfoPanel.RowHeightSpec + self.yOffset
                x2 = self.width
                y2 = (y2idx + 0.5)*self.InfoPanel.RowHeightKurs + self.yOffset
                self.dc.DrawLine(x1, y1, x2, y2)
        event.Skip()

#class ConnectionBox(wx.Panel):
#    """Create a tick box for drawing connection lines or not."""
#    def __init__(self, parent):
#        wx.Panel.__init__(self, parent)
#        self.parent = parent
#        self.InfoPanel = self.parent.InfoPanel
#        
#        self.CheckBox = wx.CheckBox(self, -1, 'Visa kopplingar')
#        self.CheckBox.SetValue(True)
#        self.CheckBox.Bind(wx.EVT_CHECKBOX, self.MyUpdate)
#
#    def MyUpdate(self, event):
#        self.InfoPanel.DrawConnections = self.CheckBox.GetValue()
#        if self.InfoPanel.DrawConnections:
#            self.InfoPanel.ConnectionPanel1.Show()
#            self.InfoPanel.ConnectionPanel2.Show()
#        else:
#            self.InfoPanel.ConnectionPanel1.Hide()
#            self.InfoPanel.ConnectionPanel2.Hide()
#        self.InfoPanel.MyUpdate()
#        event.Skip()

class QuitPanel(wx.Panel):
    """Create a panel with only a quit button."""
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        QuitButton = wx.Button(self, label="Quit")

class ListPanel(wx.Panel):
    """This class contains a panel with a list and title."""
    def __init__(self, parent, title, list, Type):
        """Initialize the panel with sizers."""
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.InfoPanel = parent.InfoPanel

        self.Title = wx.StaticText(self, -1, title)
        self.Title.SetFont(wx.Font(16, -1, wx.FONTSTYLE_NORMAL, 
                                    wx.FONTWEIGHT_BOLD))
        self.Lista = {}
        for k in list:
            self.Lista[k] = ListItem(self, k, list[k], Type)

        self.vSizer = wx.BoxSizer(wx.VERTICAL)
        self.vSizer.Add(self.Title)
        for ListEntry in sorted(self.Lista):
            self.vSizer.Add(self.Lista[ListEntry], 0, wx.ALIGN_LEFT | wx.ALL)
        self.SetSizer(self.vSizer)

class ListItem(wx.Panel):
    def __init__(self, parent, Key, Content, Type):
        """Set up a list item."""
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.InfoPanel = parent.InfoPanel

        self.Key = Key
        self.Type = Type
        if self.Type == 'Spec':
            self.Content = Content
            width = self.InfoPanel.RowWidthSpec
            height = self.InfoPanel.RowHeightSpec
        elif self.Type == 'Kurs':
            self.Content = Key + ' ' + Content
            width = self.InfoPanel.RowWidthKurs
            height = self.InfoPanel.RowHeightKurs
        elif self.Type == 'Inst':
            self.Content = Content
            width = self.InfoPanel.RowWidthInst
            height = self.InfoPanel.RowHeightInst
        self.Text = wx.StaticText(self, -1, self.Content, size=(width, height))
        self.Text.Bind(wx.EVT_LEFT_DOWN, self.ClickEvent)
        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.hSizer.Add(self.Text)
        self.SetSizer(self.hSizer)

    def ClickEvent(self, event=None):
        """Update infopanel based on mouse click."""
        self.InfoPanel.SortKey = self.Key
        self.InfoPanel.SortType = self.Type
        self.InfoPanel.MyUpdate()
        event.Skip()

if __name__ == '__main__':
    app = wx.PySimpleApp()
    frame = MyFrame(None, title='lth-specialiseringar')
    frame.Maximize()
    frame.Show()
    app.MainLoop()
